/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file module_s3.c
 * Implementation of the S3 DSI
 */

#include "globus_i_s3.h"
#include "globus_gridftp_server_s3_version.h"

#define GLOBUS_L_S3 "s3"

static
int
globus_l_s3_activate(void);

static
int
globus_l_s3_deactivate(void);

static
globus_gfs_storage_iface_t              globus_l_s3_dsi_iface =
{
    .descriptor                       = GLOBUS_GFS_DSI_DESCRIPTOR_BLOCKING
                                      | GLOBUS_GFS_DSI_DESCRIPTOR_SENDER,
    .init_func                        = globus_i_s3_start,
    .destroy_func                     = globus_i_s3_destroy,
    .send_func                        = globus_i_s3_send,
    .recv_func                        = globus_i_s3_recv,
    .command_func                     = globus_i_s3_command,
    .stat_func                        = globus_i_s3_stat
};

GlobusExtensionDefineModule(globus_gridftp_server_s3) =
{
    .module_name                      = "globus_gridftp_server_s3",
    .activation_func                  = globus_l_s3_activate,
    .deactivation_func                = globus_l_s3_deactivate,
    .version                          = &local_version
};


static
int
globus_l_s3_activate(void)
{
    char                               *config_string = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    int                                 rc;

    globus_gridftp_server_get_config_string(NULL, &config_string);

    if (config_string == NULL)
    {
        result = globus_eval_path(
                "${sysconfdir}/globus/globus-gridftp-server-s3.conf",
                &config_string);
        if (result != GLOBUS_SUCCESS)
        {
            goto done;
        }
    }

    result = globus_i_s3_config_parse(config_string, NULL);
    if (result != GLOBUS_SUCCESS)
    {
        goto parse_config_fail;
    }

    result = globus_module_activate(GLOBUS_GRIDFTP_SERVER_S3_BASE_MODULE);

    if (result != GLOBUS_SUCCESS)
    {
        goto activate_fail;
    }
    rc = globus_extension_registry_add(
        GLOBUS_GFS_DSI_REGISTRY,
        GLOBUS_L_S3,
        GlobusExtensionMyModule(globus_gridftp_server_s3),
        &globus_l_s3_dsi_iface);

    if (rc != GLOBUS_SUCCESS)
    {
        globus_module_deactivate(GLOBUS_GRIDFTP_SERVER_S3_BASE_MODULE);
    }
activate_fail:
parse_config_fail:
done:
    free(config_string);

    GlobusS3ExitResult(result);
    return result;
}
/* globus_l_s3_activate() */

static
int
globus_l_s3_deactivate(void)
{
    globus_extension_registry_remove(GLOBUS_GFS_DSI_REGISTRY, GLOBUS_L_S3);

    return globus_module_deactivate(GLOBUS_GRIDFTP_SERVER_S3_BASE_MODULE);
}
/* globus_l_s3_deactivate() */
// vim: ft=c:
